<?php

/**
 * Run old CGI applications through PHP
 * @author Adrian Kousz
 * @link https://wiki.archlinux.org/index.php/Awstats Source
 */

// Prepare environment

$newenv = $_SERVER;
$xScriptFilename = $_SERVER["X_SCRIPT_FILENAME"];
$xScriptName = $_SERVER["X_SCRIPT_NAME"];
$newenv["SCRIPT_FILENAME"] = $xScriptFilename;
$newenv["SCRIPT_NAME"] = $xScriptName;

// Check file

if (!is_executable($xScriptFilename) || is_dir($xScriptFilename)) {
	$err = 'Not Found';
	header("$_SERVER[SERVER_PROTOCOL] 404 $err");
	echo($err);
	error_log("$err: $xScriptFilename, or not allowed to execute");
	exit;
}

// Open process

$descriptorspec = array(
	array('pipe', 'r'),
	array('pipe', 'w'),
	array('pipe', 'w'),
);
$pipes = array();
$process = proc_open($xScriptFilename, $descriptorspec, $pipes, null, $newenv);

if (!is_resource($process)) {
	$err = 'Internal Server Error';
	header("$_SERVER[SERVER_PROTOCOL] 500 $err");
	echo($err);
	error_log("$err in $xScriptFilename");
	exit;
}

// All ok, send response

while (true) {
	$head = fgets($pipes[1]);
	if ($head === PHP_EOL) break;
	header($head);
}
fpassthru($pipes[1]);

// Log errors

$scriptErrors = '';
while ($line = fgets($pipes[2])) $scriptErrors .= $line;
$scriptErrors = trim($scriptErrors);
if ($scriptErrors) error_log($scriptErrors);

// Close

foreach ($pipes as $fh) fclose($fh);
proc_close($process);
